@@ -85,11 +85,11 @@ pub fn impl_tag(args: TokenStream, input: TokenStream) -> TokenStream {
 					fn from(inp: &'a #input_ident) -> Self {
 						Self {
 							title: inp.title(),
-							artists: inp.artists_vec(),
+							artist: inp.artist(),
 							year: inp.year().map(|y| y as i32),
 							album: Album::new(
 								inp.album_title(),
-								inp.album_artists_vec(),
+								inp.album_artist(),
 								inp.album_covers(),
 							),
 							track_number: inp.track_number(),
@@ -109,18 +109,12 @@ pub fn impl_tag(args: TokenStream, input: TokenStream) -> TokenStream {
 						if let Some(v) = inp.title() {
 							tag.set_title(v)
 						}
-						if let Some(v) = inp.artists_as_string() {
+						if let Some(v) = inp.artist() {
 							tag.set_artist(&v)
 						}
 						if let Some(v) = inp.year {
 							tag.set_year(v)
 						}
-						if let Some(v) = inp.album().title {
-							tag.set_album_title(v)
-						}
-						if let Some(v) = inp.album().artists {
-							tag.set_album_artist(&v.join("/"))
-						}
 						if let Some(v) = inp.track_number() {
 							tag.set_track_number(v)
 						}
@@ -134,6 +128,21 @@ pub fn impl_tag(args: TokenStream, input: TokenStream) -> TokenStream {
 							tag.set_total_discs(v)
 						}
 
+						let album = inp.album();
+
+						if let Some(v) = album.title {
+							tag.set_album_title(v)
+						}
+						if let Some(v) = album.artist {
+							tag.set_album_artist(v)
+						}
+						if let Some(v) = album.covers.0 {
+							tag.set_front_cover(v)
+						}
+						if let Some(v) = album.covers.1 {
+							tag.set_back_cover(v)
+						}
+
 						tag
 					}
 				}
@@ -171,3 +180,91 @@ pub fn impl_tag(args: TokenStream, input: TokenStream) -> TokenStream {
 		.to_compile_error()
 		.into()
 }
+
+#[proc_macro]
+pub fn str_accessor(input: TokenStream) -> TokenStream {
+	let input_str = input.to_string();
+	let name = input_str.replace("_", " ");
+
+	format!(
+		"/// Returns the {display}
+			fn {ident}(&self) -> Option<&str> {{
+				None
+			}}
+			/// Sets the {display}
+			fn set_{ident}(&mut self, _{ident}: &str) {{}}
+			/// Removes the {display}
+			fn remove_{ident}(&mut self) {{}}
+			",
+		ident = input_str,
+		display = name,
+	)
+	.parse()
+	.expect("Unable to parse str accessor:")
+}
+
+#[proc_macro]
+pub fn u16_accessor(input: TokenStream) -> TokenStream {
+	let input_str = input.to_string();
+	let name = input_str.replace("_", " ");
+
+	format!(
+		"/// Returns the {display}
+			fn {ident}(&self) -> Option<u16> {{
+				None
+			}}
+			/// Sets the {display}
+			fn set_{ident}(&mut self, _{ident}: u16) {{}}
+			/// Removes the {display}
+			fn remove_{ident}(&mut self) {{}}
+			",
+		ident = input_str,
+		display = name,
+	)
+		.parse()
+		.expect("Unable to parse u16 accessor:")
+}
+
+#[proc_macro]
+pub fn u32_accessor(input: TokenStream) -> TokenStream {
+	let input_str = input.to_string();
+	let name = input_str.replace("_", " ");
+
+	format!(
+		"/// Returns the {display}
+			fn {ident}(&self) -> Option<u32> {{
+				None
+			}}
+			/// Sets the {display}
+			fn set_{ident}(&mut self, _{ident}: u32) {{}}
+			/// Removes the {display}
+			fn remove_{ident}(&mut self) {{}}
+			",
+		ident = input_str,
+		display = name,
+	)
+		.parse()
+		.expect("Unable to parse u32 accessor:")
+}
+
+#[proc_macro]
+pub fn i32_accessor(input: TokenStream) -> TokenStream {
+	let input_str = input.to_string();
+	let name = input_str.replace("_", " ");
+
+	format!(
+		"/// Returns the {display}
+			fn {ident}(&self) -> Option<i32> {{
+				None
+			}}
+			/// Sets the {display}
+			fn set_{ident}(&mut self, _{ident}: i32) {{}}
+			/// Removes the {display}
+			fn remove_{ident}(&mut self) {{}}
+			",
+		ident = input_str,
+		display = name,
+	)
+		.parse()
+		.expect("Unable to parse i32 accessor:")
+}
\ No newline at end of file
